package ru.landgrafhomyak.bgtu.computer_graphics1

import kotlin.math.PI
import kotlin.math.cos
import kotlin.math.sin
import kotlin.math.sqrt
import ru.landgrafhomyak.bgtu.computer_graphics1.model_builder.ModelTriangles
import ru.landgrafhomyak.bgtu.computer_graphics1.model_builder.Point2F
import ru.landgrafhomyak.bgtu.computer_graphics1.model_builder.Polygon
import ru.landgrafhomyak.bgtu.computer_graphics1.model_builder.TextureType

private fun rot_n(degrees: Int, length: Float): Point2F {
	val radians = degrees.toLong().toDouble() * PI / 180;
	return Point2F(
		x = cos(radians).times(length).toFloat(),
		y = sin(radians).times(length).toFloat()
	)
}


val model = ModelTriangles.fromPolygons(
	Polygon(
		TextureType.HSV,
		Polygon.PolygonVertex(0f, 1f, 1f, rot_n(0, 1f)),
		Polygon.PolygonVertex(-1f, 1f, 1f, rot_n(0, 2f)),
		Polygon.PolygonVertex(-1f, -1f, 1f, rot_n(300, 2f * sqrt(2f))),
		Polygon.PolygonVertex(1f, -1f, 1f, rot_n(240, 2f)),
		Polygon.PolygonVertex(1f, 0f, 1f, rot_n(240, 1f)),
	),
	Polygon(
		TextureType.HSV,
		Polygon.PolygonVertex(0f, 1f, 1f, rot_n(0, 1f)),
		Polygon.PolygonVertex(-1f, 1f, 1f, rot_n(0, 2f)),
		Polygon.PolygonVertex(-1f, 1f, -1f, rot_n(60, 2f * sqrt(2f))),
		Polygon.PolygonVertex(1f, 1f, -1f, rot_n(120, 2f)),
		Polygon.PolygonVertex(1f, 1f, 0f, rot_n(120, 1f))
	),
	Polygon(
		TextureType.HSV,
		Polygon.PolygonVertex(1f, 0f, 1f, rot_n(240, 1f)),
		Polygon.PolygonVertex(1f, -1f, 1f, rot_n(240, 2f)),
		Polygon.PolygonVertex(1f, -1f, -1f, rot_n(180, 2f * sqrt(2f))),
		Polygon.PolygonVertex(1f, 1f, -1f, rot_n(120, 2f)),
		Polygon.PolygonVertex(1f, 1f, 0f, rot_n(120, 1f))
	),
	Polygon(
		TextureType.HSV,
		Polygon.PolygonVertex(1f, 0f, 1f, rot_n(240, 1f)),
		Polygon.PolygonVertex(1f, 1f, 0f, rot_n(120, 1f)),
		Polygon.PolygonVertex(0f, 1f, 1f, rot_n(0, 1f))
	),
	Polygon(
		TextureType.GRADIENT_CHESS,
		Polygon.PolygonVertex(-1f, -1f, 1f, 0f, 0f),
		Polygon.PolygonVertex(-1f, -1f, -1f, 0f, 1f),
		Polygon.PolygonVertex(-1f, 1f, -1f, 1f, 1f),
		Polygon.PolygonVertex(-1f, 1f, 1f, 1f, 0f),
	),
	Polygon(
		TextureType.CHESS,
		Polygon.PolygonVertex(1f, -1f, -1f, 0f, 1f),
		Polygon.PolygonVertex(-1f, -1f, -1f, 0f, 0f),
		Polygon.PolygonVertex(-1f, 1f, -1f, 1f, 0f),
		Polygon.PolygonVertex(1f, 1f, -1f, 1f, 1f),
	),
	Polygon(
		TextureType.WHITE_TRANSPARENCY,
		Polygon.PolygonVertex(-1f, -1f, 1f, 0f, 1f),
		Polygon.PolygonVertex(1f, -1f, 1f, 1f, 1f),
		Polygon.PolygonVertex(1f, -1f, -1f, 1f, 0f),
		Polygon.PolygonVertex(-1f, -1f, -1f, 0f, 0f),
	)
)