package ru.landgrafhomyak.bgtu.computer_graphics1.model_builder


class ModelTriangles(
	val modelVertexes: FloatArray,
	val textureVertexes: FloatArray,
	val textureTypes: Array<TextureType>
) {
	companion object {
		private fun <T> MutableCollection<T>.addAll(vararg elems: T) {
			this.addAll(elems)
		}

		fun fromPolygons(vararg polygons: Polygon): ModelTriangles {
			val modelVertexes = ArrayList<Float>()
			val textureVertexes = ArrayList<Float>()
			val textureTypes = ArrayList<TextureType>()

			polygons@ for (polygon in polygons) {
				val it = polygon.vertexes.iterator()
				if (!it.hasNext()) continue@polygons
				val root = it.next()
				if (!it.hasNext()) continue@polygons
				var prev = it.next()
				for (curr in it) {
					modelVertexes.addAll(
						root.modelVertex.x, root.modelVertex.y, root.modelVertex.z,
						prev.modelVertex.x, prev.modelVertex.y, prev.modelVertex.z,
						curr.modelVertex.x, curr.modelVertex.y, curr.modelVertex.z,
					)
					textureVertexes.addAll(
						root.textureVertex.x, root.textureVertex.y,
						prev.textureVertex.x, prev.textureVertex.y,
						curr.textureVertex.x, curr.textureVertex.y,
					)
					textureTypes.addAll(
						polygon.textureType, polygon.textureType, polygon.textureType
					)

					prev = curr
				}
			}
			return ModelTriangles(
				modelVertexes = modelVertexes.toTypedArray().toFloatArray(),
				textureVertexes = textureVertexes.toTypedArray().toFloatArray(),
				textureTypes = textureTypes.toTypedArray()
			)
		}
	}
}