package ru.landgrafhomyak.bgtu.computer_graphics1

import ru.landgrafhomyak.bgtu.computer_graphics1.model_builder.TextureType

internal val vertexShaderSource = """
	#version 300 es
	
	uniform mat4 transform;
	uniform mat4 projection;
	uniform vec2 screen_scale;
	in vec3 vertex_pos;
	in vec2 texture_pos;
	in int texture_type;
	out vec2 texture_pos_v2f;
	flat out int texture_type_v2f;
	
	void main() {
		gl_Position = vec4(vertex_pos, 1.0);
		gl_Position *= transform;
		gl_Position *= projection;
		gl_Position.xy *= screen_scale;
	    texture_pos_v2f = texture_pos;
	    texture_type_v2f = texture_type;
	}
""".trimIndent()

internal val fragmentShaderSource = """
	#version 300 es
	#define PI 3.1415926535897932384626433832795
	
	precision mediump float;
	in  vec2 texture_pos_v2f;
	flat in int texture_type_v2f;
	out vec4 FragColor;
	
	vec3 hsv2rgb(float, float, float);
	vec3 hsv_texture(float, float);
	float atan_gradient(float k, float radius, float lo, float hi);
	float linear_gradient(float k, float lo, float hi);
	
	void main() {
		switch (texture_type_v2f) {
			case ${TextureType.WHITE_TRANSPARENCY.ordinal}:
				FragColor = vec4(1.0, 1.0, 1.0, 0.5);
				break;
			case ${TextureType.CHESS.ordinal}:
 				if (int( round(floor(texture_pos_v2f.x / 0.1)) + round(floor(texture_pos_v2f.y / 0.1)) ) % 2 == 0) {
					FragColor = vec4(0.0, 0.0, 0.0, 1.0);
				} else {
					FragColor = vec4(1.0, 1.0, 1.0, 1.0);
				}
				break;
			case ${TextureType.HSV.ordinal}:
				FragColor = vec4(hsv_texture(texture_pos_v2f.x, texture_pos_v2f.y), 1.0);
				break;
			case ${TextureType.GRADIENT_CHESS.ordinal}:
			{
				float radius = 0.2;
				if (int( round(floor((texture_pos_v2f.x - 0.5) / 0.033)) + round(floor((texture_pos_v2f.y - 0.5) / 0.033)) ) % 2 == 0) {
					FragColor = vec4(
						0.0 + linear_gradient(texture_pos_v2f.x, 0.0, 0.5),
						0.0 + linear_gradient(texture_pos_v2f.y, 0.0, 0.5),
						0.5 - min(linear_gradient(texture_pos_v2f.x, 0.0, 0.35) + linear_gradient(texture_pos_v2f.y, 0.0, 0.35), 0.5), 
						1.0
					);
				} else {
					FragColor = vec4(
						0.0 + linear_gradient(texture_pos_v2f.x, 0.0, 1.0),
						0.0 + linear_gradient(texture_pos_v2f.y, 0.0, 1.0),
						1.0 - min(linear_gradient(texture_pos_v2f.x, 0.0, 0.8) + linear_gradient(texture_pos_v2f.y, 0.0, 0.8), 1.0), 
						1.0
					);
				}
				break;
			}
			default:
				FragColor = vec4(0.0, 0.0, 0.0, 0.0);
		}
	}
	
	vec3 hsv_texture(float x, float y) {
		float angle = atan(y, x);
		return hsv2rgb(mod(mod(angle / PI  * 180.0, 360.0) + 360.0, 360.0), 1.0, 1.0);
	}
	
	float atan_gradient(float k, float radius, float lo, float hi) {
		return ((atan((k - 0.5) * 2.0 * radius) / atan(radius)) + 0.5) * (hi - lo) + lo;
	}
	float linear_gradient(float k, float lo, float hi) {
		return k * (hi - lo) + lo;
	}
	vec3 hsv2rgb(float hue, float saturation, float value) {
		int hi = int(floor(hue / 60.0)) % 6;
		float f = hue / 60.0 - floor(hue / 60.0);
		float v = value;
		float p = value * (1.0 - saturation);
		float q = value * (1.0 - f * saturation);
		float t = value * (1.0 - (1.0 - f) * saturation);
		switch (hi) {
			case 0:
				return vec3(v, t, p);
			case 1:
				return vec3(q, v, p);
			case 2:
				return vec3(p, v, t);
			case 3:
				return vec3(p, q, v);
			case 4:
				return vec3(t, p, v);
			default:
				return vec3(v, p, q);
		}
	}
""".trimIndent()